
#TODO: return also the probabilities

build_multimodel <- function(data, recipe) {
  
  memb_dim <- 'ensemble'
  time_dim <- 'syear'
  
  if (tolower(recipe$Analysis$Datasets$Multimodel$approach) == 'pooled') {
    
    browser()
    
    skill_metrics <- list()

    for (met in recipe$Analysis$Workflow$Skill$metric) {  
      
      if(met == 'enscorr'){

        ## Calculate ensemble mean
        ensemble_mean <- Apply(data$hcst$data, target_dim = c(memb_dim, time_dim),
                               function(...) {
                                 res <- abind(..., along = 1)
                                 res <- mean(res)},
                               ncores = recipe$Analysis$ncores)$output1

        ## Calculate correlation
        skill <- s2dv::Corr(ensemble_mean, data$obs$data,
                              dat_dim = 'dat',
                              time_dim = time_dim,
                              method = 'pearson',
                              memb_dim = memb_dim,
                              memb = memb,
                              conf = F,
                              pval = F,
                              sign = T,
                              alpha = 0.05,
                              ncores = ncores)
          skill <- lapply(skill, function(x) {
            .drop_dims(x)})
          skill_metrics[[ metric ]] <- skill$corr

        }
      
      }  
      
      if(met == 'enssprerr'){
        
        # Remove ensemble dim from obs to avoid veriApply warning
        obs_noensdim <- ClimProjDiags::Subset(data$obs$data, "ensemble", 1,
                                              drop = "selected")
        capture.output(
          skill <- easyVerification::veriApply(verifun = 'EnsSprErr',
                                               fcst = data$hcst$data,
                                               obs = obs_noensdim,
                                               tdim = which(names(dim(data$hcst$data))==time_dim),
                                               ensdim = which(names(dim(data$hcst$data))==memb_dim),
                                               na.rm = na.rm,
                                               ncpus = ncores)
        )
        remove(obs_noensdim)
        skill <- .drop_dims(skill)
        skill_metrics[[ metric ]] <- skill
      }
        
        
    
    
    
    
    
  } else if (tolower(recipe$Analysis$Datasets$Multimodel$approach) %in% c('mean','median')) {
    
    # Probabilistic hindcast and forecast
    stop('Probabilities for multi-model mean is still under development.
          If Skill is used, the results are not correct for the probabilitic metrics.
          Probabilities cannot be computed because data has an extra dimension (model).
          Also, the function should return the observed probabilities.
          Maybe it is better to use GetProbs.')
    prob <- NULL #Probabilities(recipe, data)
    
    # Deterministic hindcast
    data$hcst$data <- multiApply::Apply(data = data$hcst$data, 
                                        target_dims = 'ensemble',
                                        fun = tolower(recipe$Analysis$Datasets$Multimodel$approach),
                                        na.rm = TRUE,
                                        ncores = recipe$Analysis$ncores)$output1
    data$hcst$data <- multiApply::Apply(data = data$hcst$data,
                                        target_dims = 'model',
                                        fun = tolower(recipe$Analysis$Datasets$Multimodel$approach),
                                        na.rm = FALSE,
                                        ncores = recipe$Analysis$ncores)$output1
    data$hcst$dims <- s2dv::InsertDim(data = data$hcst$dims, posdim = 6, lendim = 1, name = 'ensemble')
    data$hcst$dims <- dim(data$hcst$data)
    
    # Deterministic forecast
    if (!is.null(recipe$Analysis$Time$fcst_year)) {
      data$fcst$data <- multiApply::Apply(data = data$fcst$data, 
                                          target_dims = 'ensemble',
                                          fun = tolower(recipe$Analysis$Datasets$Multimodel$approach),
                                          na.rm = TRUE,
                                          ncores = recipe$Analysis$ncores)$output1
      data$fcst$data <- multiApply::Apply(data = data$fcst$data,
                                          target_dims = 'model',
                                          fun = tolower(recipe$Analysis$Datasets$Multimodel$approach),
                                          na.rm = FALSE,
                                          ncores = recipe$Analysis$ncores)$output1
      data$fcst$dims <- s2dv::InsertDim(data = data$fcst$dims, posdim = 6, lendim = 1, name = 'ensemble')
      data$fcst$dims <- dim(data$fcst$data)
    }
    
  } else {stop('Incorrect multi-model approach')}
  
  return(list(data = data, 
              prob = prob))
}
